@@ -1,11 +1,13 @@
-import six
-
 import os
 import re
 import ctypes
 import PIL.Image
+import tempfile
 
 from django.core.exceptions import ImproperlyConfigured
+from django.core.files.storage import default_storage
+from django.utils.encoding import force_bytes
+from django.utils import six
 
 from cropduster.files import ImageFile
 from cropduster.utils import json
@@ -29,6 +31,9 @@ except:
 
 if not libxmp:
     check_file_format = get_format_info = None
+
+    def file_to_dict(f):
+        return {}
 else:
     try:
         exempi = libxmp._exempi
@@ -47,6 +52,11 @@ else:
     if not get_format_info.restype:
         get_format_info.restype = ctypes.c_bool
 
+    try:
+        from libxmp.utils import file_to_dict
+    except ImportError:
+        from libxmp import file_to_dict
+
 
 class EnumerationMeta(type):
 
@@ -140,7 +150,7 @@ def file_format_supported(file_path):
         raise IOError("File %s could not be found" % file_path)
     if not check_file_format:
         return False
-    file_format = check_file_format(os.path.abspath(file_path))
+    file_format = check_file_format(force_bytes(os.path.abspath(file_path)))
     format_options = ctypes.c_int()
     if file_format != FileFormats.XMP_FT_UNKNOWN:
         format_options = get_format_info(
@@ -169,8 +179,13 @@ class MetadataDict(dict):
     """
 
     def __init__(self, file_path):
-        self.file_path = file_path
-        ns_dict = libxmp.file_to_dict(file_path)
+        self.tmp_file = tempfile.NamedTemporaryFile()
+        with default_storage.open(file_path) as f:
+            self.tmp_file.write(f.read())
+            self.tmp_file.flush()
+            self.tmp_file.seek(0)
+        self.file_path = self.tmp_file.name
+        ns_dict = file_to_dict(self.file_path)
         self.clean(ns_dict)
 
     def clean(self, ns_dict):
@@ -215,6 +230,8 @@ class MetadataDict(dict):
                     v = re.sub(r'^xmp\.did:', '', v).lower()
                 elif ns_prefix == 'crop' and k == 'json':
                     v = json.loads(v)
+                elif ns_prefix == 'crop' and k == 'md5':
+                    v = v.lower()
 
                 m = re.search(r'^(.*)\[(\d+)\](?=\/|\Z)', k)
                 if m:
@@ -292,4 +309,51 @@ class MetadataImageFile(ImageFile):
     def __init__(self, *args, **kwargs):
         super(MetadataImageFile, self).__init__(*args, **kwargs)
         if self:
-            self.metadata = MetadataDict(self.path)
+            self.metadata = MetadataDict(self.name)
+
+
+def get_xmp_from_file(file_path):
+    return libxmp.XMPFiles(file_path=file_path).get_xmp()
+
+
+def get_xmp_from_bytes(img_bytes):
+    tmp = tempfile.NamedTemporaryFile(delete=False)
+    tmp.write(img_bytes)
+    tmp.close()
+    try:
+        return get_xmp_from_file(tmp.name)
+    finally:
+        os.unlink(tmp.name)
+
+
+def get_xmp_from_storage(file_path, storage=default_storage):
+    with storage.open(file_path, mode='rb') as f:
+        return get_xmp_from_bytes(f.read())
+
+
+def put_xmp_to_file(xmp_meta, file_path):
+    xmp_file = libxmp.XMPFiles(file_path=file_path, open_forupdate=True)
+
+    if not xmp_file.can_put_xmp(xmp_meta):
+        if not file_format_supported(file_path):
+            raise Exception("Image format of %s does not allow metadata" % (file_path))
+        else:
+            raise Exception("Could not add metadata to image %s" % (file_path))
+
+    xmp_file.put_xmp(xmp_meta)
+    xmp_file.close_file()
+
+
+def put_xmp_to_storage(xmp_meta, file_path, storage=default_storage):
+    tmp = tempfile.NamedTemporaryFile(delete=False)
+    try:
+        with default_storage.open(file_path, mode='rb') as f:
+            tmp.write(f.read())
+        tmp.close()
+        put_xmp_to_file(xmp_meta, tmp.name)
+        with open(tmp.name, mode='rb') as f:
+            data = f.read()
+        with storage.open(file_path, mode='wb') as f:
+            f.write(data)
+    finally:
+        os.unlink(tmp.name)
