@@ -1,7 +1,5 @@
 from __future__ import division
 
-import six
-
 import os
 import hashlib
 
@@ -9,17 +7,15 @@ import PIL.Image
 
 from django import forms
 from django.core.exceptions import ObjectDoesNotExist
+from django.core.files.storage import default_storage
 from django.conf import settings
 from django.forms.forms import NON_FIELD_ERRORS
 from django.forms.models import BaseModelFormSet
-from django.forms.util import ErrorDict as _ErrorDict
+from django.forms.utils import ErrorDict as _ErrorDict
+from django.utils.encoding import force_text
 from django.utils.html import conditional_escape
 from django.utils.safestring import mark_safe
-
-try:
-    from django.utils.encoding import force_unicode
-except ImportError:
-    from django.utils.encoding import force_text as force_unicode
+from django.utils import six
 
 from cropduster.models import Thumb
 from cropduster.utils import (json, get_upload_foldername, get_min_size,
@@ -34,7 +30,7 @@ class ErrorDict(_ErrorDict):
         for k, v in self.items():
             if k == NON_FIELD_ERRORS:
                 k = ''
-            error_list.append(u'%s%s' % (k, conditional_escape(force_unicode(v))))
+            error_list.append(u'%s%s' % (k, conditional_escape(force_text(v))))
 
         return mark_safe(u'<ul class="errorlist">%s</ul>'
                 % ''.join([u'<li>%s</li>' % e for e in error_list]))
@@ -47,7 +43,7 @@ def clean_upload_data(data):
         pil_image = PIL.Image.open(image)
     except IOError as e:
         if e.errno:
-            error_msg = force_unicode(e)
+            error_msg = force_text(e)
         else:
             error_msg = u"Invalid or unsupported image file"
         raise forms.ValidationError({"image": [error_msg]})
@@ -55,6 +51,7 @@ def clean_upload_data(data):
         extension = get_image_extension(pil_image)
 
     upload_to = data['upload_to'] or None
+
     folder_path = get_upload_foldername(image.name, upload_to=upload_to)
 
     (w, h) = (orig_w, orig_h) = pil_image.size
@@ -81,13 +78,14 @@ def clean_upload_data(data):
     # File is good, get rid of the tmp file
     orig_file_path = os.path.join(folder_path, 'original' + extension)
     image.seek(0)
-    image_contents = image.read()
-    with open(os.path.join(settings.MEDIA_ROOT, orig_file_path), 'wb+') as f:
-        f.write(image_contents)
     md5_hash = hashlib.md5()
-    md5_hash.update(image_contents)
+    default_storage.save(orig_file_path, image)
+    with default_storage.open(orig_file_path) as f:
+        md5_hash.update(f.read())
+        f.seek(0)
+        data['image'] = f
     data['md5'] = md5_hash.hexdigest()
-    data['image'] = open(os.path.join(settings.MEDIA_ROOT, orig_file_path), mode='rb')
+
     return data
 
 
@@ -144,15 +142,15 @@ class CropForm(forms.Form):
         css = {'all': (
             u"%scropduster/css/cropduster.css?v=5" % settings.STATIC_URL,
             u"%scropduster/css/jquery.jcrop.css?v=4" % settings.STATIC_URL,
-            u"%scropduster/css/upload.css?v=5" % settings.STATIC_URL,
+            u"%scropduster/css/upload.css?v=6" % settings.STATIC_URL,
         )}
         js = (
             u"%scropduster/js/json2.js" % settings.STATIC_URL,
             u"%scropduster/js/jquery.class.js" % settings.STATIC_URL,
             u"%scropduster/js/jquery.form.js?v=1" % settings.STATIC_URL,
-            u"%scropduster/js/jquery.jcrop.js?v=4" % settings.STATIC_URL,
-            u"%scropduster/js/upload.js?v=11" % settings.STATIC_URL,
-            u"%scropduster/js/cropduster.js?v=6" % settings.STATIC_URL,
+            u"%scropduster/js/jquery.jcrop.js?v=5" % settings.STATIC_URL,
+            u"%scropduster/js/cropduster.js?v=8" % settings.STATIC_URL,
+            u"%scropduster/js/upload.js?v=16" % settings.STATIC_URL,
         )
 
     image_id = forms.IntegerField(required=False)
